-- Preparation

USE Playground;

-- Demo tables for some of the demos
DROP TABLE IF EXISTS dbo.Fruits;
DROP TABLE IF EXISTS dbo.Colors;

CREATE TABLE dbo.Fruits
    (FruitNum  int         NOT NULL,
     FruitName varchar(35) NOT NULL);
CREATE TABLE dbo.Colors
    (ColorNum  int         NOT NULL,
     ColorName varchar(35) NOT NULL,
     ColorCode binary(3)   NOT NULL);

CREATE CLUSTERED INDEX cix_Fruits_FruitNum
ON dbo.Fruits(FruitNum)
CREATE NONCLUSTERED INDEX cix_Fruits_FruitName
ON dbo.Fruits(FruitName)

CREATE CLUSTERED INDEX cix_Colors_ColorNum
ON dbo.Colors(ColorNum)
CREATE NONCLUSTERED INDEX cix_Colors_ColorName
ON dbo.Colors(ColorName)


INSERT INTO dbo.Fruits (FruitNum,
                        FruitName)
VALUES (1, 'Apple'),
       (3, 'Cherry'),
       (2, 'Lime'),
       (3, 'Melon'),
       (2, 'Orange');

INSERT INTO dbo.Colors (ColorNum,
                        ColorName,
                        ColorCode)
VALUES (4, 'Blue', 0x0000FF),
       (2, 'Orange', 0xFFA500),
       (1, 'Red', 0xFF0000),
       (2, 'Yellow', 0xFFFF00);


-- Demo 1

USE AdventureWorks2017;

SELECT p.BusinessEntityID,
       x.AvgQty
FROM   (SELECT TOP (1) BusinessEntityID FROM Person.Person) AS p
CROSS APPLY
       (SELECT AVG (sod.OrderQty) AS AvgQty
        FROM   Sales.SalesOrderDetail AS sod)               AS x
OPTION (FORCE ORDER, NO_PERFORMANCE_SPOOL);


-- Demo 2

USE AdventureWorks2017;

SELECT     at.Name,
           cr.Name
FROM       Person.AddressType   AS at
CROSS JOIN Person.CountryRegion AS cr;

-- Second window

SELECT     sod.SalesOrderID,
           sod.SalesOrderDetailID,
           sod.OrderQty,
           sod.ProductID,
           soh.OrderDate,
           soh.DueDate,
           soh.ShipDate
FROM       Sales.SalesOrderDetail AS sod
INNER JOIN Sales.SalesOrderHeader AS soh
   ON      soh.SalesOrderID = soh.SalesOrderID;



-- Demo 3

USE Playground;

SELECT     f.FruitNum,
           f.FruitName,
           c.ColorNum,
           c.ColorName
FROM       dbo.Colors AS c
INNER JOIN dbo.Fruits AS f
   ON      f.FruitNum = c.ColorNum
ORDER BY   c.ColorNum;

SELECT     f.FruitNum,
           f.FruitName,
           c.ColorNum,
           c.ColorName
FROM       dbo.Colors AS c
INNER JOIN dbo.Fruits AS f
   ON      f.FruitNum = c.ColorNum
ORDER BY   c.ColorName;

-- Second window

SELECT     f.FruitNum,
           f.FruitName,
           c.ColorNum,
           c.ColorName
FROM       dbo.Colors AS c
INNER JOIN dbo.Fruits AS f
   ON      f.FruitNum = c.ColorNum
ORDER BY   f.FruitName
--OPTION (FORCE ORDER);


-- Demo 4

USE Playground;

SELECT     c.ColorNum,
           c.ColorName,
           c.ColorCode,
           f.FruitNum,
           f.FruitName
FROM       dbo.Colors AS c
INNER JOIN dbo.Fruits AS f
   ON      f.FruitNum = c.ColorNum
--OPTION (LOOP JOIN);


-- Demo 5

USE AdventureWorks2017;

SELECT   sp.BusinessEntityID,
         sp.TerritoryID,
         (SELECT MAX (soh.TaxAmt)
          FROM   Sales.SalesOrderHeader AS soh
          WHERE  soh.TerritoryID = sp.TerritoryID) AS MaxTax
FROM     Sales.SalesPerson AS sp
WHERE    sp.BusinessEntityID BETWEEN 280 AND 285
ORDER BY sp.TerritoryID;
