-- Demo 1

USE AdventureWorks2017;

SELECT    sod.SalesOrderID,
          sod.SalesOrderDetailID,
          sod.OrderQty,
          sod.ProductID,
          soh.DueDate,
          soh.ShipDate,
          soh.ModifiedDate
FROM      Sales.SalesOrderDetail AS sod
LEFT JOIN Sales.SalesOrderHeader AS soh
  ON      soh.ModifiedDate = sod.ModifiedDate


-- Demo 2

USE AdventureWorks2017;

SET STATISTICS IO ON;
DECLARE @n int = 100;
SELECT SUM (CAST (CHECKSUM (*) AS bigint))
FROM   Sales.SalesOrderDetail  AS sod
INNER JOIN
       (SELECT   TOP (@n) *
        FROM     Sales.SalesOrderDetail
        ORDER BY SalesOrderID) AS sod2
   ON sod2.ModifiedDate = sod.ModifiedDate
OPTION (HASH JOIN, MAXDOP 1,
        OPTIMIZE FOR (@n = 10000)); -- Hint used to force bad estimate

SET STATISTICS IO ON;
DECLARE @n int = 10000;
SELECT SUM (CAST (CHECKSUM (*) AS bigint))
FROM   Sales.SalesOrderDetail  AS sod
INNER JOIN
       (SELECT   TOP (@n) *
        FROM     Sales.SalesOrderDetail
        ORDER BY SalesOrderID) AS sod2
   ON sod2.ModifiedDate = sod.ModifiedDate
OPTION (HASH JOIN, MAXDOP 1,
        OPTIMIZE FOR (@n = 100));   -- Hint used to force bad estimate

-- Second window

DBCC FREEPROCCACHE;


-- Demo 3

SELECT     c.StoreID,
           c.TerritoryID,
           soh.DueDate,
           soh.ShipDate
FROM       Sales.Customer         AS c
INNER JOIN Sales.SalesOrderHeader AS soh
   ON      CLR_database.dbo.Delay (c.CustomerID, 1)
         = CLR_database.dbo.Delay (soh.CustomerID, 1);


-- Demo 4

USE AdventureWorks2017;

DECLARE @n int = 20000;
SELECT     c.StoreID,
           c.TerritoryID,
           soh.SalesOrderID,
           soh.ShipDate
FROM      (SELECT TOP (@n) * FROM Sales.Customer ORDER BY CustomerID) AS c
INNER JOIN Sales.SalesOrderHeader                                     AS soh
   ON      c.CustomerID = soh.CustomerID
--ORDER BY   soh.SalesOrderID
OPTION (OPTIMIZE FOR (@n = 20000), HASH JOIN);


-- Demo 5

USE AdventureWorks2017;

SELECT soh.SalesOrderID,
       soh.ModifiedDate
FROM   Sales.SalesOrderHeader AS soh
UNION
SELECT a.StateProvinceID,
       a.ModifiedDate
FROM   Person.Address AS a
--OPTION (HASH UNION);

-- Second window

SELECT soh.CustomerID,
       soh.ModifiedDate
FROM   Sales.SalesOrderHeader AS soh
UNION
SELECT a.StateProvinceID,
       a.ModifiedDate
FROM   Person.Address AS a
OPTION (HASH UNION);

SELECT a.StateProvinceID,
       a.ModifiedDate
FROM   Person.Address AS a
UNION
SELECT soh.CustomerID,
       soh.ModifiedDate
FROM   Sales.SalesOrderHeader AS soh
OPTION (HASH UNION);


-- Demo 6

USE AdventureWorks2017;

SELECT     soh1.OrderDate,
           soh2.DueDate
FROM       Sales.SalesOrderHeader AS soh1
INNER JOIN Sales.SalesOrderHeader AS soh2
   ON      soh2.ModifiedDate = soh1.ModifiedDate;

SELECT     soh1.OrderDate,
           soh2.DueDate
FROM       Sales.SalesOrderHeader AS soh1
INNER JOIN Sales.SalesOrderHeader AS soh2
   ON      soh2.CustomerID = soh1.CustomerID;

SELECT     soh1.OrderDate,
           soh2.DueDate
FROM       Sales.SalesOrderHeader AS soh1
INNER JOIN Sales.SalesOrderHeader AS soh2
   ON      soh2.RevisionNumber  = soh1.RevisionNumber
   AND     soh2.Status          = soh1.Status
   AND     soh2.OnlineOrderFlag = soh1.OnlineOrderFlag;

SELECT     soh1.OrderDate,
           soh2.DueDate
FROM       Sales.SalesOrderHeader AS soh1
INNER JOIN Sales.SalesOrderHeader AS soh2
   ON      soh2.RevisionNumber  = soh1.RevisionNumber
   AND     soh2.Status          = soh1.Status
   AND     soh2.OnlineOrderFlag = soh1.OnlineOrderFlag
   AND     soh2.ModifiedDate    > soh1.ModifiedDate
OPTION (MAXDOP 1);
